@@ -3,9 +3,82 @@ package Clustericious::Client;
 use strict; no strict 'refs';
 use warnings;
 
-# ABSTRACT: Construct command line and perl clients for RESTful services.
-our $VERSION = '0.83_01'; # VERSION
+our $VERSION = '0.64';
 
+=head1 NAME
+
+Clustericious::Client - Constructor for clients of Clustericious apps.
+
+=head1 SYNOPSIS
+
+ package Foo::Client;
+
+ use Clustericious::Client;
+
+ route 'welcome' => '/';                   # GET /
+
+ route status;                             # GET /status
+
+ route myobj => [ 'MyObject' ];            # GET /myobj
+
+ route something => GET => '/some/';
+
+ route remove => DELETE => '/something/';
+
+ object 'obj';                             # Defaults to /obj
+
+ object 'foo' => '/something/foo';         # Can override the URL
+
+ route status => \"Get the status";        # Scalar refs are documentation
+
+ route_doc status => "Get the status";     # or you can use route_doc
+
+ route_meta status => { auto_failover => 1 } # route_meta sets attributes of routes
+
+ ----------------------------------------------------------------------
+
+ use Foo::Client;
+
+ my $f = Foo::Client->new();
+ my $f = Foo::Client->new(server_url => 'http://someurl');
+ my $f = Foo::Client->new(app => 'MyApp'); # For testing...
+
+ my $welcome = $f->welcome();              # GET /
+
+ my $status = $f->status();                # GET /status
+
+ my $myobj = $f->myobj('key');             # GET /myobj/key, MyObject->new()
+
+ my $something = $f->something('this');    # GET /some/this
+
+ $f->remove('foo');                        # DELETE /something/foo
+
+ my $obj = $f->obj('this', 27);            # GET /obj/this/27
+ # Returns either 'Foo::Client::Obj' or 'Clustericious::Client::Object'
+
+ $f->obj({ set => 'this' });               # POST /obj
+
+ $f->obj('this', 27, { set => 'this' });   # POST /obj/this/27
+
+ $f->obj_delete('this', 27);               # DELETE /obj/this/27
+
+ my $obj = $f->foo('this');                # GET /something/foo/this
+
+=head1 DESCRIPTION
+
+Some very simple helper functions with a clean syntax to build a REST
+type client suitable for Clustericious applications.
+
+The builder functions add methods to the client object that translate
+into basic REST functions.  All of the 'built' methods return undef on
+failure of the REST/HTTP call, and auto-decode the returned body into
+a data structure if it is application/json.
+
+Using Clustericious::Client also allows for easy creation of both
+perl APIs and command line clients for a L<Clustericious> server.
+See L<Clustericious::Client::Command>.
+
+=cut
 
 use Mojo::Base qw/-base/;
 
@@ -20,7 +93,37 @@ use Clustericious::Client::Meta::Route;
 use MojoX::Log::Log4perl;
 use Log::Log4perl qw/:easy/;
 use File::Temp;
+use Proc::Daemon;
 
+=head1 ATTRIBUTES
+
+This class inherits from L<Mojo::Base>, and handles attributes like
+that class.  The following additional attributes are used.
+
+=over
+
+=item C<client>
+
+A client to process the HTTP stuff with.  Defaults to a
+L<Mojo::UserAgent>.
+
+=item C<app>
+
+For testing, you can specify a Mojolicious app name.
+
+=item C<server_url>
+
+You can override the URL prefix for the client, otherwise it
+will look it up in the config file.
+
+=item C<res>
+
+After an HTTP error, the built methods return undef.  This function
+will return the L<Mojo::Message::Response> from the server.
+
+res->code and res->message are the returned HTTP code and message.
+
+=cut
 
 has server_url => '';
 has [qw(tx res userinfo client)];
@@ -37,7 +140,6 @@ sub import
     push @{"${caller}::ISA"}, $class unless $caller->isa($class);
     *{"${caller}::route"} = \&route;
     *{"${caller}::route_meta"} = \&route_meta;
-    *{"${caller}::route_args"} = \&route_args;
     *{"${caller}::route_doc"} = sub {
         Clustericious::Client::Meta->add_route( $caller, @_ )
     };
@@ -46,6 +148,38 @@ sub import
     }
 }
 
+=back
+
+=head1 METHODS
+
+=over
+
+=item C<new>
+
+ my $f = Foo::Client->new();
+ my $f = Foo::Client->new(server_url => 'http://someurl');
+ my $f = Foo::Client->new(app => 'MyApp'); # For testing...
+
+If the configuration file has a "url" entry, this will
+be used as the default url (first case above).  Additionally,
+if the ssh_tunnel key is given in the config file, a tunnel
+may be created automatically (and destroyed when the client
+is destroyed).  Here's a sample configuration :
+
+   "url" : "http://localhost:12345",
+   "ssh_tunnel" : {
+        "remote_host" : "omidev",
+        "server_host" : "localhost",
+        "server_port" : "9014"
+    },
+
+This would automatically execute this
+
+    ssh -N -L12345:localhost:9014 omidev
+
+in the background.
+
+=cut
 
 sub new
 {
@@ -56,9 +190,8 @@ sub new
     {
         my $app = $self->{app};
         $app = $app->new() unless ref($app);
-        my $client = Mojo::UserAgent->new;
-        return undef unless $client;
-        eval { $client->server->app($app) } // $client->app($app);
+        my $client = Mojo::UserAgent->new(app => $app)
+            or return undef;
 
         $self->client($client);
     }
@@ -67,18 +200,49 @@ sub new
         $self->client(Mojo::UserAgent->new);
         if (not length $self->server_url)
         {
-            my $url = $self->_config->url;
-            $url =~ s{/$}{};
-            $self->server_url($url);
+            $self->server_url($self->_config->url);
         }
     }
 
     $self->client->inactivity_timeout($ENV{CLUSTERICIOUS_KEEP_ALIVE_TIMEOUT} || 300);
+    if (!$args{server_url} && $self->_config->ssh_tunnel(default => '')) {
+        INFO "Found an ssh tunnel for ".(ref $self)." in config file";
+        $self->_start_ssh_tunnel;
+    }
 
     return $self;
 }
 
+=item tx
 
+The most recent HTTP::Transaction.
+
+=cut
+
+=item userinfo
+
+Credentials currently stored.
+
+=cut
+
+=item remote
+
+Tell the client to use the remote information in the configuration.
+For instance, if the config has
+
+ remotes :
+    test :
+        url: http://foo
+    bar :
+        url: http://baz
+        username : one
+        password : two
+
+Then setting remote("test") uses the first
+url, and setting remote("bar") uses the
+second one.
+
+=cut
 
 sub remote {
     my $self = shift;
@@ -96,6 +260,11 @@ sub remote {
     $self->_remote($remote);
 }
 
+=item remotes
+
+Return a list of available remotes.
+
+=cut
 
 sub remotes {
     my $self = shift;
@@ -103,6 +272,16 @@ sub remotes {
     return keys %found;
 }
 
+=item C<login>
+
+Log in to the server.  This will send basic auth info
+along with every subsequent request.
+
+    $f->login; # looks for username and password in $app.conf
+    $f->login("elmer", "fudd");
+    $f->login(username => "elmer", password => "fudd");
+
+=cut
 
 sub login {
     my $self = shift;
@@ -114,8 +293,18 @@ sub login {
     $self->userinfo(join ':', $user,$pw);
 }
 
+=item C<errorstring>
 
-sub errorstring {
+After an error, this returns an error string made up of the server
+error code and message.  (use res->code and res->message to get the
+parts)
+
+(e.g. "Error: (500) Internal Server Error")
+
+=cut
+
+sub errorstring
+{
     my $self = shift;
     WARN "Missing response in client object" unless $self->res;
     return unless $self->res;
@@ -124,18 +313,41 @@ sub errorstring {
       || sprintf( "(%d) %s", $self->res->code, $self->res->message );
 }
 
+=back
 
-sub has_error {
-    my $c = shift;
-    return unless $c->tx || $c->res;
-    return 1 if $c->tx && $c->tx->error;
-    return 1 if $c->res && !$c->res->is_status_class(200);
-    return 0;
-}
+=head1 FUNCTIONS
 
+=over
 
+=item C<route>
 
-sub route {
+ route 'subname';                    # GET /subname
+ route subname => '/url';            # GET /url
+ route subname => GET => '/url';     # GET /url
+ route subname => POST => '/url';    # POST /url
+ route subname => DELETE => '/url';  # DELETE /url
+ route subname => ['SomeObjectClass'];
+ route subname \"<documentation> <for> <some> <args>";
+
+Makes a method subname() that does the REST action.  Any scalar
+arguments are tacked onto the end of the url separated by a slash.
+If any argument begins with "--", it and its successor are treated
+as part of URL query string (for a GET request).  If any argument
+begins with a single "-", it and it successor are treated as HTTP
+headers to send (for a GET request).  If you pass a hash
+reference, the method changes to POST and the hash is encoded into
+the body as application/json.
+
+A hash reference after a POST method becomes headers.
+
+A scalar reference as the final argument adds documentation
+about this route which will be displayed by the command-line
+client.
+
+=cut
+
+sub route
+{
     my $subname = shift;
     my $objclass = ref $_[0] eq 'ARRAY' ? shift->[0] : undef;
     my $doc      = ref $_[-1] eq 'SCALAR' ? ${ pop() } : "";
@@ -148,30 +360,40 @@ sub route {
             route_name => $subname
     );
 
-    $meta->set(method => $method);
-    $meta->set(url    => $url);
     $meta->set_doc($doc);
 
-    if ($objclass) {
+    if ($objclass)
+    {
         eval "require $objclass";
         if ($@) {
             LOGDIE "Error loading $objclass : $@" unless $@ =~ /Can't locate/i;
         }
-    }
 
+        no strict 'refs';
+        *{caller() . "::$subname"} =
+        sub
+        {
+            my $self = shift;
+            $objclass->new($self->_doit($meta,$method,$url,@_), $self);
+        };
+    }
+    else
     {
         no strict 'refs';
-        *{caller() . "::$subname"} = sub {
-            my $self = shift;
-            my @args = $self->meta_for($subname)->process_args(@_);
-            my $got = $self->_doit($meta,$method,$url,@args);
-            return $objclass->new($got, $self) if $objclass;
-            return $got;
-        };
+        *{caller() . "::$subname"} = sub { shift->_doit($meta,$method,$url,@_); };
     }
 
 }
 
+=item route_meta
+
+Set metadata attributes for this route.
+
+    route_meta 'bucket_map' => { auto_failover => 1 }
+    route_meta 'bucket_map' => { quiet_post => 1 }
+    route_meta 'bucket_map' => { skip_existing => 1 }
+
+=cut
 
 sub route_meta {
     my $name = shift;
@@ -184,19 +406,30 @@ sub route_meta {
     $meta->set($_ => $attrs->{$_}) for keys %$attrs;
 }
 
+=item C<object>
 
-sub route_args {
-    my $name = shift;
-    my $args = shift;
-    die "args must be an array ref" unless ref $args eq 'ARRAY';
-    my $meta = Clustericious::Client::Meta::Route->new(
-        client_class => scalar caller(),
-        route_name   => $name
-    );
+ object 'objname';                   # defaults to URL /objname
+ object objname => '/some/url';
 
-    $meta->set(args => $args);
-}
+Creates two methods, one named with the supplied objname() (used for
+create, retrieve, update), and one named objname_delete().
 
+Any scalar arguments to the created functions are tacked onto the end
+of the url.  Performs a GET by default, but if you pass a hash
+reference, the method changes to POST and the hash is encoded into the
+body as application/json.
+
+The 'object' routes will automatically look for a class named with the
+object name, but upper case first letter and first after any
+underscores, which are removed:
+
+ object 'myobj';    Foo::Client::Myobj;
+ object 'my_obj';   Foo::Client::MyObj;
+
+If such a class isn't found, object will default to returning a
+L<Clustericious::Client::Object>.
+
+=cut
 
 sub object {
     my $objname = shift;
@@ -232,7 +465,6 @@ sub object {
             client_class => $caller,
             route_name   => $objname.'_delete',
         );
-        $meta->set(dont_read_files => 1);
         shift->_doit( $meta, DELETE => $url, @_ );
     };
     *{"${caller}::${objname}_search"} = sub {
@@ -240,7 +472,6 @@ sub object {
             client_class => $caller,
             route_name   => $objname.'_search'
         );
-        $meta->set(dont_read_files => 1);
         shift->_doit( $meta, POST => "$url/search", @_ );
     };
 }
@@ -268,89 +499,47 @@ sub _doit {
 
     $url = Mojo::URL->new($url) unless ref $url;
     my $parameters = $url->query;
-
-    # Set up mappings from parameter names to modifier callbacks.
-    my %url_modifier;
-    my %payload_modifer;
-    my %gen_url_modifier = (
-        query  => sub { my $name = shift;
-            sub { my ($u,$v) = @_; $u->query({$name => $v}) }  },
-        append => sub { my $name = shift;
-            sub { my ($u,$v) = @_; push @{ $u->path->parts } , $v; $u; } },
-    );
-    my %gen_payload_modifier = (
-        array => sub {
-            my ( $name, $key ) = @_;
-            LOGDIE "missing key for array payload modifier" unless $key;
-            sub { my $body = shift; $body ||= {}; push @{ $body->{$key} }, ( $name => shift ); $body; }
-        },
-        hash => sub {
-            my $name = shift;
-            sub { my $body = shift; $body ||= {}; $body->{$name} = shift; $body; }
-        },
-    );
-    if ($meta && (my $arg_spec = $meta->get('args'))) {
-        for (@$arg_spec) {
-            my $name = $_->{name};
-            if (my $modifies_url = $_->{modifies_url}) {
-                $url_modifier{$name} =
-                    ref($modifies_url) eq 'CODE'            ? $modifies_url
-                 :  ($a = $gen_url_modifier{$modifies_url}) ? $a->($name)
-                 :  die "don't understand how to interpret modifies_url=$modifies_url";
-            }
-            if (my $modifies_payload = $_->{modifies_payload}) {
-                 $payload_modifer{$name} =
-                    ref($modifies_payload) eq 'CODE' ? $modifies_payload
-                 : ($a = $gen_payload_modifier{$modifies_payload}) ? $a->($name,$_->{key})
-                 : LOGDIE "don't understand how to interpret modifies_payload=$modifies_payload";
-            }
-        }
-    }
-
-    while (defined(my $arg = shift @args)) {
+    while (defined(my $arg = shift @args))
+    {
         if (ref $arg eq 'HASH') {
             $method = 'POST';
             $parameters->append(skip_existing => 1) if $meta && $meta->get("skip_existing");
             $body = encode_json $arg;
             $headers = { 'Content-Type' => 'application/json' };
-        } elsif (ref $arg eq 'CODE') {
+        }
+        elsif (ref $arg eq 'CODE')
+        {
             $cb = $self->_mycallback($arg);
-        } elsif (my $code = $url_modifier{$arg}) {
-            $url = $code->($url, shift @args);
-        } elsif (my $code2 = $payload_modifer{$arg}) {
-            $body = $code2->($body, shift @args);
-        } elsif ($method eq "GET" && $arg =~ s/^--//) {
+        }
+        elsif ($method eq "GET" && $arg =~ s/^--//) {
             my $value = shift @args;
             $parameters->append($arg => $value);
-        } elsif ($method eq "GET" && $arg =~ s/^-//) {
+        }
+        elsif ($method eq "GET" && $arg =~ s/^-//) {
             # example: $client->esdt(-range => [1 => 100]);
             my $value = shift @args;
             if (ref $value eq 'ARRAY') {
                 $value = "items=$value->[0]-$value->[1]";
             }
             $headers->{$arg} = $value;
-        } elsif ($method eq "POST" && !ref $arg) {
+        }
+        elsif ($method eq "POST" && !ref $arg) {
             $body = $arg;
             $headers = shift @args if $args[0] && ref $args[0] eq 'HASH';
-        } else {
+        }
+        else
+        {
             push @{ $url->path->parts }, $arg;
         }
     }
     $url = $url->to_abs unless $url->is_abs;
-    WARN "url $url is not absolute" unless $url =~ /^http/i || $ENV{HARNESS_ACTIVE};
+    WARN "url $url is not absolute" unless $url =~ /^http/i;
 
     $url->userinfo($self->userinfo) if $self->userinfo;
 
     DEBUG ( (ref $self)." : $method " ._sanitize_url($url));
     $headers->{Connection} ||= 'Close';
     $headers->{Accept}     ||= 'application/json';
-
-    if($body && ref $body eq 'HASH' || ref $body eq 'ARRAY')
-    {
-        $headers->{'Content-Type'} = 'application/json';
-        $body = encode_json $body;
-    }
-
     return $self->client->build_tx($method, $url, $headers, $body, $cb) if $cb;
 
     my $tx = $self->client->build_tx($method, $url, $headers, $body);
@@ -360,11 +549,9 @@ sub _doit {
     $self->res($res);
     $self->tx($tx);
 
-    my $auth_header;
-    if (($tx->res->code||0) == 401 && ($auth_header = $tx->res->headers->www_authenticate)
-        && !$url->userinfo && ($self->_has_auth || $self->_can_auth)) {
+    if (($tx->res->code||0) == 401 && !$url->userinfo && ($self->_has_auth || $self->_can_auth)) {
         DEBUG "received code 401, trying again with credentials";
-        my ($realm) = $auth_header =~ /realm=(.*)$/i;
+        my ($realm) = $tx->res->headers->www_authenticate =~ /realm=(.*)$/i;
         my $host = $url->host;
         $self->login( $self->_has_auth ? () : $self->_get_user_pw($host,$realm) );
         return $self->_doit($meta ? $meta : (), @_);
@@ -377,7 +564,7 @@ sub _doit {
             "text/plain";
         };
         return $method =~ /HEAD|DELETE/       ? 1
-            : $content_type =~ qr[application/json] ? decode_json($res->body)
+            : $content_type eq 'application/json' ? decode_json($res->body)
             : $res->body;
     }
 
@@ -437,7 +624,7 @@ sub _mycallback
 
         if ($tx->res->is_status_class(200))
         {
-            my $body = $tx->res->headers->content_type =~ qr[application/json]
+            my $body = $tx->res->headers->content_type eq 'application/json'
                 ? decode_json($tx->res->body) : $tx->res->body;
 
             $cb->($body ? $body : 1);
@@ -498,29 +685,50 @@ sub _get_user_pw  {
     my $self = shift;
     my $host = shift;
     my $realm = shift;
-    $realm = '' unless defined $realm;
     return @{ $self->_cache->{$host}{$realm} } if exists($self->_cache->{$host}{$realm});
     # "use"ing causes too many warnings; load on demand.
-    require Term::Prompt;
-    my $user = Term::Prompt::prompt('x', "Username for $realm at $host : ", '', $ENV{USER} // $ENV{USERNAME});
-    my $pw = Term::Prompt::prompt('p', 'Password:', '', '');
+    require IO::Prompt;
+    my $user = IO::Prompt::prompt("Username for $realm at $host : ");
+    my $pw = IO::Prompt::prompt("Password : ",{-echo=>'*'});
     $self->_cache->{$host}{$realm} = [ $user, $pw ];
     return ($user,$pw);
 }
 
+=item meta_for
+
+Get the metadata for a route.
+
+    $client->meta_for('welcome');
+
+Returns a Clustericious::Client::Meta::Route object.
+
+=cut
 
 sub meta_for {
     my $self = shift;
-    my $route_name = shift || [ caller 1 ]->[3];
-    if ( $route_name =~ /::([^:]+)$/ ){
-        $route_name = $1;
-    }
+    my $route_name = shift;
     my $meta = Clustericious::Client::Meta::Route->new(
         route_name   => $route_name,
         client_class => ref $self
     );
 }
 
+=back
+
+=head1 COMMON ROUTES
+
+These are routes that are automatically supported by all clients.
+See L<Clustericious::RouteBuilder::Common>.  Each of these
+must also be in L<Clustericious::Client::Meta> for there
+to be documentation.
+
+=over
+
+=item version
+
+Retrieve the version on the server.
+
+=cut
 
 sub version {
     my $self = shift;
@@ -529,6 +737,11 @@ sub version {
     $self->_doit($meta, GET => '/version');
 }
 
+=item status
+
+Retrieve the status from the server.
+
+=cut
 
 sub status {
     my $self = shift;
@@ -537,6 +750,11 @@ sub status {
     $self->_doit($meta, GET => '/status');
 }
 
+=item api
+
+Retrieve the API from the server
+
+=cut
 
 sub api {
     my $self = shift;
@@ -545,6 +763,11 @@ sub api {
     $self->_doit( $meta, GET => '/api' );
 }
 
+=item logtail
+
+Get the last N lines of the server log file.
+
+=cut
 
 sub logtail {
     my $self = shift;
@@ -552,490 +775,99 @@ sub logtail {
     return { text => $got };
 }
 
+sub _ssh_pidfile {
+    sprintf("%s/%s_acps_ssh.pid",($ENV{TMPDIR} || "/tmp"),shift->_appname);
+}
 
-1;
+=item ssh_tunnel_is_up
 
-__END__
-=pod
-
-=head1 NAME
-
-Clustericious::Client - Construct command line and perl clients for RESTful services.
-
-=head1 VERSION
-
-version 0.83_01
-
-=head1 SYNOPSIS
-
-tracks.pm :
-
-    package Tracks;
-    use Clustericious::Client;
-
-    route 'mixes' => '/mixes.json';
-    route_doc mixes => 'Get a list of mixes.';
-    route_args mixes => [
-        { name => 'api_key', type => '=s', modifies_url => "query", required => 1 },
-        { name => 'per_page',type => '=i', modifies_url => "query", },
-        { name => 'tags',    type => '=s', modifies_url => "query" },
-    ];
-    # a 'mixes' method will be constructed automatically.
-    # a 'mixes' command line parameter will be recognized automatically.
-
-    route 'play' => '/play.json';
-    route_args play => [
-        { name => 'token', type => '=s', modifies_url => 'query', required => 1 }
-    ];
-    sub play {
-        my $c = shift;
-        my %args = $c->meta_for->process_args(@_);
-        # do something with $args{token}
-    }
-    # A 'play' command line parameter will call the above method.
-
-tracks.pl :
-
-    #!/usr/bin/env perl
-
-    use lib '.';
-    use Log::Log4perl qw/:easy/;
-    Log::Log4perl->easy_init($TRACE);
-    use tracks;
-
-    my $t = Tracks->new(server_url => 'http://8tracks.com' );
-    my $mixes = $t->mixes(
-         tags => 'jazz',
-         api_key => $api_key,
-         per_page => 2,
-         ) or die $t->errorstring;
-    print "Mix : $_->{name}\n" for @{ $mixes->{mixes} };
-
-tracks_cli :
-
-    #!/usr/bin/env perl
-
-    use lib '.';
-    use Clustericious::Client::Command;
-    use tracks;
-
-    use Log::Log4perl qw/:easy/;
-    Log::Log4perl->easy_init($TRACE);
-
-    Clustericious::Client::Command->run(Tracks->new, @ARGV);
-
-~/etc/Tracks.conf :
-
-    ---
-    url : 'http://8tracks.com'
-
-From the command line :
-
-    $ perl tracks.pl
-    $ tracks_cli mixes --api_key foo --tags jazz
-
-=head1 DESCRIPTION
-
-This is a beta release.  The API is subject to changes without notice.
-
-Clustericious::Client is library for construction clients for RESTful
-services.  It provides a mapping between command line arguments, method
-arguments, and URLs.
-
-The builder functions add methods to the client object that translate
-into basic REST functions.  All of the 'built' methods return undef on
-failure of the REST/HTTP call, and auto-decode the returned body into
-a data structure if it is application/json.
-
-=head1 ATTRIBUTES
-
-This class inherits from L<Mojo::Base>, and handles attributes like
-that class.  The following additional attributes are used.
-
-=head2 client
-
-A client to process the HTTP stuff with.  Defaults to a
-L<Mojo::UserAgent>.
-
-=head2 app
-
-For testing, you can specify a Mojolicious app name.
-
-=head2 server_url
-
-You can override the URL prefix for the client, otherwise it
-will look it up in the config file.
-
-=head2 res, tx
-
-After an HTTP error, the built methods return undef.  This function
-will return the L<Mojo::Message::Response> from the server.
-
-res->code and res->message are the returned HTTP code and message.
-
-tx has the Mojo::Transaction::HTTP object.
-
-=head1 METHODS
-
-=head2 new
-
- my $f = Foo::Client->new();
- my $f = Foo::Client->new(server_url => 'http://someurl');
- my $f = Foo::Client->new(app => 'MyApp'); # For testing...
-
-If the configuration file has a "url" entry, this will
-be used as the default url (first case above).
-
-=head2 userinfo
-
-Credentials currently stored.
-
-=head2 remote
-
-Tell the client to use the remote information in the configuration.
-For instance, if the config has
-
- remotes :
-    test :
-        url: http://foo
-    bar :
-        url: http://baz
-        username : one
-        password : two
-
-Then setting remote("test") uses the first
-url, and setting remote("bar") uses the
-second one.
-
-=head2 remotes
-
-Return a list of available remotes.
-
-=head2 login
-
-Log in to the server.  This will send basic auth info
-along with every subsequent request.
-
-    $f->login; # looks for username and password in $app.conf
-    $f->login("elmer", "fudd");
-    $f->login(username => "elmer", password => "fudd");
-
-=head2 errorstring
-
-After an error, this returns an error string made up of the server
-error code and message.  (use res->code and res->message to get the
-parts)
-
-(e.g. "Error: (500) Internal Server Error")
-
-=head2 has_error
-
-Returns true if there was a recent error.
-
-=head1 FUNCTIONS
-
-=head2 route
-
- route 'subname';                    # GET /subname
- route subname => '/url';            # GET /url
- route subname => GET => '/url';     # GET /url
- route subname => POST => '/url';    # POST /url
- route subname => DELETE => '/url';  # DELETE /url
- route subname => ['SomeObjectClass'];
- route subname \"<documentation> <for> <some> <args>";
- route_args subname => [ { name => 'param', type => "=s", modifies_url => 'query' } ]
- route_args subname => [ { name => 'param', type => "=i", modifies_url => 'append' } ]
-
-Makes a method subname() that does the REST action.
-
- route subname => $url => $doc
-
-is equivalent to
-
- route      subname => $url
- route_args subname => [ { name => 'all', positional => 'many', modifies_url => 'append' } ];
- route_doc  subname => $$doc
-
-with the additional differences that GET becomes a POST if the argument is
-a hashref, and heuristics are used to read YAML files and STDIN.
-
-See route_args and route_doc below.
-
-=head2 route_meta
-
-Set metadata attributes for this route.
-
-    route_meta 'bucket_map' => { auto_failover => 1 }
-    route_meta 'bucket_map' => { quiet_post => 1 }
-    route_meta 'bucket_map' => { skip_existing => 1 }
-
-=head2 route_args
-
-Set arguments for this route.  This allows command line options
-to be transformed into method arguments, and allows normalization
-and validation of method arguments.  route_args associates an array
-ref with the name of a route.  Each entry in the array ref is a hashref
-which may have keys as shown in this example :
-
-  route_args send => [
-            {
-                name     => 'what',              # name of the route
-                type     => '=s',                # type (see L<Getopt::Long>)
-                alt      => 'long|extra|big',    # alternative names
-                required => 0,                   # Is it required?
-                doc      => 'get a full status', # brief documentation
-            },
-            {
-                name     => 'items',               # name of the route
-                type     => '=s',                  # type (see L<Getopt::Long>)
-                doc      => 'send a list of items' # brief docs
-                preprocess => 'list'               # make an array ref from a list
-            },
-        ];
-
-The keys have the following effect :
-
-=over
-
-=item name
-
-The name of the option.  This should be preceded by two dashes
-on the command line.  It is also sent as the named argument to the
-method call.
-
-=item type
-
-A type, as described in L<Getopt::Long>.  This will be appended to
-the name to form the option specification.
-
-=item alt
-
-An alternative name or names (joined by |).
-
-=item required
-
-If this arg is required, set this to 1.
-
-=item doc
-
-A brief description to be printed in error messages and help documentation.
-
-=item preprocess
-
-Can be either C<yamldoc>, C<list> or C<datetime>.
-
-For yamldoc and list, the argument is expected
-to refer to either a filename which exists, or else "-" for STDIN.  The contents
-are then transformed from YAML (for yamldoc), or split on carriage returns (for list)
-to form either a data structure or an arrayref, respectively.
-
-For datetime the string is run through Date::Parse and turned into an ISO 8601 datetime.
-
-=item modifies_url
-
-Describes how the URL is affected by the arguments.  Can be
-'query', 'append', or a code reference.
-
-'query' adds to the query string, e.g.
-
-    route subname '/url'
-    route_args subname => [ { name => 'foo', type => "=s", modifies_url => 'query' } ]
-
-This will cause this invocation :
-
-    $foo->subname( "foo" => "bar" )
-
-to send a GET request to /url?foo=bar.
-
-Similarly, 'append' is equivalent to
-
-    sub { my ($u,$v) = @_; push @{ $u->path->parts } , $v }
-
-i.e. append the parameter to the end of the URL path.
-
-If route_args is omitted for a route, then arguments with a '--'
-are treated as part of the query string, and arguments with a '-'
-are treated as HTTP headers (for a GET request).  If a hash
-reference is passed, the method changes to POST and the hash is
-encoded into the body as application/json.
-
-=item modifies_payload, key
-
-Describes how the parameter modifies the payload.
-
-'hash' means set $body->{$name} to $value.
-'array' means push ( $name => $value ) onto $body->{$key}.
-   (key should also be specified)
-
-=item positional
-
-Can be 'one' or 'many'.
-
-If set, this is a positional parameter, not a named parameter.  i.e.
-getopt will not be used to parse the command line, and
-it will be take from a list sent to the method.  For instance
-
-  route_args name => [ { name => 'id', positional => 'one' } ];
-
-Then
-
-  $client->name($id)
-
-or
-
- commandlineclient name id
-
-will result in the method receiving (id => $id).
-
-If set to 'many', multiple parameters may be sent, e.g.
-
-    $client->name($id1, $id2,....)
-
-=back
-
-=head2 object
-
- object 'objname';                   # defaults to URL /objname
- object objname => '/some/url';
-
-Creates two methods, one named with the supplied objname() (used for
-create, retrieve, update), and one named objname_delete().
-
-Any scalar arguments to the created functions are tacked onto the end
-of the url.  Performs a GET by default, but if you pass a hash
-reference, the method changes to POST and the hash is encoded into the
-body as application/json.
-
-The 'object' routes will automatically look for a class named with the
-object name, but upper case first letter and first after any
-underscores, which are removed:
-
- object 'myobj';    Foo::Client::Myobj;
- object 'my_obj';   Foo::Client::MyObj;
-
-If such a class isn't found, object will default to returning a
-L<Clustericious::Client::Object>.
-
-=head2 meta_for
-
-Get the metadata for a route.
-
-    $client->meta_for('welcome');
-
-Returns a Clustericious::Client::Meta::Route object.
-
-=head1 COMMON ROUTES
-
-These are routes that are automatically supported by all clients.
-See L<Clustericious::RouteBuilder::Common>.
-
-=head2 version
-
-Retrieve the version on the server.
-
-=head2 status
-
-Retrieve the status from the server.
-
-=head2 api
-
-Retrieve the API from the server
-
-=head2 logtail
-
-Get the last N lines of the server log file.
-
-=head1 EXAMPLES
-
- package Foo::Client;
- use Clustericious::Client;
-
- route 'welcome' => '/';                   # GET /
- route status;                             # GET /status
- route myobj => [ 'MyObject' ];            # GET /myobj
- route something => GET => '/some/';
- route remove => DELETE => '/something/';
-
- object 'obj';                             # Defaults to /obj
- object 'foo' => '/something/foo';         # Can override the URL
-
- route status => \"Get the status";        # Scalar refs are documentation
- route_doc status => "Get the status";     # or you can use route_doc
- route_args status => [                    # route_args sets method or cli arguments
-            {
-                name     => 'full',
-                type     => '=s',
-                required => 0,
-                doc      => 'get a full status',
-            },
-        ];
-
- route_args wrinkle => [                   # methods correspond to "route"s
-     {
-         name => 'time'
-     }
- ];
-
- sub wrinkle {                             # provides cli command as well as a method
-    my $c = shift;
-    my %args = @_;
-    if ($args{time}) {
-            ...
-    }
- }
-
- ----------------------------------------------------------------------
-
- use Foo::Client;
-
- my $f = Foo::Client->new();
- my $f = Foo::Client->new(server_url => 'http://someurl');
- my $f = Foo::Client->new(app => 'MyApp'); # For testing...
-
- my $welcome = $f->welcome();              # GET /
- my $status = $f->status();                # GET /status
- my $myobj = $f->myobj('key');             # GET /myobj/key, MyObject->new()
- my $something = $f->something('this');    # GET /some/this
- $f->remove('foo');                        # DELETE /something/foo
-
- my $obj = $f->obj('this', 27);            # GET /obj/this/27
- # Returns either 'Foo::Client::Obj' or 'Clustericious::Client::Object'
-
- $f->obj({ set => 'this' });               # POST /obj
- $f->obj('this', 27, { set => 'this' });   # POST /obj/this/27
- $f->obj_delete('this', 27);               # DELETE /obj/this/27
- my $obj = $f->foo('this');                # GET /something/foo/this
-
- $f->status(full => "yes");
- $f->wrinkle( time => 1 ); 
-
- ----------------------
-
- #!/bin/sh
- fooclient status
- fooclient status --full yes
- fooclient wrinkle --time
-
-=head1 SEE ALSO
-
-L<Clustericious::Config>, L<Clustericious>, L<Mojolicious>
-
-=head1 AUTHOR
-
-original author: Curt Tilmes
-
-current maintainer: Graham Ollis <plicease@cpan.org>
-
-contributors:
-
-Brian Duggan
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2013 by NASA GSFC.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
+Check to see if an ssh tunnel is alive for the current client.
 
 =cut
 
+sub ssh_tunnel_is_up {
+    my $pidfile = shift->_ssh_pidfile;
+    if (-e $pidfile) {
+        my ($pid) = IO::File->new("<$pidfile")->getlines;
+        if ($pid and kill 0, $pid) {
+            DEBUG "found running ssh ($pid)";
+            return 1;
+        }
+    }
+    return 0;
+}
+
+sub _ssh_cmd {
+    my $self = shift;
+    my $conf = $self->_config->ssh_tunnel;
+    my $url = Mojo::URL->new($self->server_url);
+    my $cmd = sprintf( "ssh -n -N -L%d:%s:%d %s",
+        $url->port,           $conf->{server_host},
+        $conf->{server_port}, $conf->{remote_host}
+    );
+    return $cmd;
+}
+
+sub _start_ssh_tunnel {
+    my $self = shift;
+
+    return if $self->ssh_tunnel_is_up;
+
+    my $error_file = File::Temp->new();
+    my $out_file = File::Temp->new();
+
+    my $cmd = $self->_ssh_cmd;
+    INFO "Executing $cmd";
+    my $proc = Proc::Daemon->new(
+        exec_command => $cmd,
+        pid_file     => $self->_ssh_pidfile,
+        child_STDERR => $error_file,
+        child_STDOUT => $out_file,
+        work_dir     => "/tmp"
+    );
+    my $pid = $proc->Init or do {
+        FATAL "Could not start $cmd, see $error_file or $out_file";
+        $error_file->unlink_on_destroy(0);
+        $out_file->unlink_on_destroy(0);
+    };
+    sleep 1;
+    DEBUG "new ssh pid is $pid";
+}
+
+=item stop_ssh_tunnel
+
+Stop any running ssh tunnel for this client.
+
+=cut
+
+sub stop_ssh_tunnel {
+    my $self = shift;
+    my $proc = Proc::Daemon->new( pid_file => $self->_ssh_pidfile);
+    WARN "pid file is ".$self->_ssh_pidfile;
+    my $pid = $proc->Status($self->_ssh_pidfile) or do {
+        INFO "Could not stop ssh tunnel for ".(ref $self);
+        return;
+    };
+    my $killed = $proc->Kill_Daemon($self->_ssh_pidfile) or do {
+        WARN "Could not kill ssh for ".(ref $self);
+        return;
+    };
+    DEBUG "Killed ssh for ".(ref $self)." ($killed)";
+    return 1;
+}
+
+=back
+
+=head1 ENVIRONMENT
+
+Set ACPS_SUPPRESS_404 to a regular expression in order to
+not print messages when a 404 response is returned from urls
+matching that regex.
+
+=head1 NOTES
+
+This is a beta release.  The API is subject to changes without notice.
+
+=cut
+
+1;
