@@ -1,12 +1,53 @@
 package Clustericious::Client::Command;
 
+=head1 NAME
+
+Clustericious::Client::Command - Command line type processing for clients.
+
+=head1 SYNOPSIS
+
+# in fooclient :
+
+ use Foo::Client;
+ use Clustericious::Client::Command;
+
+ Clustericious::Client::Command->run(Foo::Client->new, @ARGV);
+
+Then
+
+ fooclient status
+ fooclient --trace root status
+ fooclient version
+ fooclient foobject 31
+ fooclient foobject_search --color beige
+
+=head1 DESCRIPTION
+
+This will try to take command line arguments and call the right client
+methods.
+
+Calling 'fooclient bar baz' is equivalent to Foo::Client->new()->bar("baz").
+
+=head1 CAVEATS
+
+There are currently a few heuristics used when one of the arguments
+is a filename (i.e. is it a yaml file that should be parsed and send
+as a hashref, or a filename that should be PUT?  Should STDIN be
+used?).  These need to be formalized and documented.
+
+=head1 NOTES
+
+This is a beta release, the API is subject to change without notice.
+
+=head1 TODO
+
+Document and stabilize the API.
+
+=cut
+
 use strict;
 use warnings;
 
-# ABSTRACT: Command line type processing for clients.
-our $VERSION = '0.83_01'; # VERSION
-
-
 use File::Basename qw/basename/;
 use YAML::XS qw(Load Dump LoadFile);
 use Log::Log4perl qw/:easy/;
@@ -14,7 +55,6 @@ use Scalar::Util qw/blessed/;
 use Data::Rmap qw/rmap_ref/;
 use File::Temp;
 
-use Clustericious::Log;
 use Clustericious::Client::Meta;
 
 sub _usage {
@@ -49,6 +89,13 @@ EOPRINT
     exit 0;
 }
 
+=head1 METHODS
+
+=head2 C<run>
+
+ Clustericious::Client::Command->run(Some::Clustericious::Client->new, @ARGV);
+
+=cut
 
 our $Ssh = "ssh -o StrictHostKeyChecking=no -o BatchMode=yes -o PasswordAuthentication=no";
 sub _expand_remote_glob {
@@ -95,18 +142,15 @@ sub _load_yaml {
 sub run {
     my $class = shift;
     my $client = shift;
-    my @args = @_ ? @_ : @ARGV;
-    our $TESTING;
 
-    return $class->_usage($client) if !$args[0] || $args[0] =~ /help/;
+    return $class->_usage($client) if !$ARGV[0] || $ARGV[0] =~ /help/;
 
-    # Preprocessing for any common args, e.g. --remote
     my $arg;
     ARG :
-    while ($arg = shift @args) {
+    while ($arg = shift @_) {
         for ($arg) {
             /--remote/ and do {
-                my $remote = shift @args;
+                my $remote = shift;
                 TRACE "Using remote $remote";
                 $client->remote($remote);
                 next ARG;
@@ -120,16 +164,16 @@ sub run {
     # Map some alternative command line forms.
     my $try_stdin;
     if ( $method eq 'create' ) {
-        $method = shift @args or $class->_usage( $client, "Missing <object>" );
+        $method = shift @_ or $class->_usage( $client, "Missing <object>" );
         $try_stdin = 1;
     }
 
     if ( $method =~ /^(delete|search)$/ ) { # e.g. search -> app_search
-        $method = ( shift @args ) . '_' . $method;
+        $method = ( shift @_ ) . '_' . $method;
     }
 
     unless ($client->can($method)) {
-        $class->_usage($client, "Unrecognized argument : $method");
+        $class->_usage($client, "Unrecognized arguments");
         return;
     }
 
@@ -138,47 +182,14 @@ sub run {
         client_class => ref $client
     );
 
-    if ($meta->get('args')) {
-        # No heuristics for args.
-
-        my $obj = $client->$method({ command_line => 1 }, @args);
-
-        ERROR $client->errorstring if $client->has_error;
-
-        # Copied from below, until that code is deprecated.
-        if ( blessed($obj) && $obj->isa("Mojo::Transaction") ) {
-            if ( my $res = $obj->success ) {
-                print $res->code," ",$res->default_message,"\n";
-            } else {
-                my ( $message, $code ) = $obj->error;
-                ERROR $code if $code;
-                ERROR $message;
-            }
-        } elsif (ref $obj eq 'HASH' && keys %$obj == 1 && $obj->{text}) {
-            print $obj->{text};
-        } elsif ($client->tx && $client->tx->req->method eq 'POST' && $meta->get("quiet_post")) {
-            my $msg = $client->res->code." ".$client->res->default_message;
-            my $got = $client->res->json;
-            if ($got && ref $got eq 'HASH' and keys %$got==1 && $got->{text}) {
-                $msg .= " ($got->{text})";
-            }
-            INFO $msg;
-        } else {
-           print _prettyDump($obj) unless $TESTING;
-        }
-        return;
-    }
-
-    # Code below here should be deprecated, these are various heuristics for argument processing.
-
     my @extra_args = ( '/dev/null' );
     my $have_filenames;
 
     # Assume we have files and/or remote globs
-    if ( !$meta->get('dont_read_files') && @args > 0 && ( -r $_[-1] || $_[-1] =~ /^\S+:/ ) ) {
+    if ( !$meta->get('dont_read_files') && @_ > 0 && ( -r $_[-1] || $_[-1] =~ /^\S+:/ ) ) {
         $have_filenames = 1;
         @extra_args = ();
-        while (my $arg = pop @args) {
+        while (my $arg = pop @_) {
             if ($arg =~ /^\S+:/) {
                 push @extra_args, _expand_remote_glob($arg);
             } elsif (-e $arg) {
@@ -187,20 +198,20 @@ sub run {
                 LOGDIE "Do not know how to interpret argument : $arg";
             }
         }
-    } elsif ( $try_stdin && (-r STDIN) && @args==0) {
+    } elsif ( $try_stdin && (-r STDIN) && @_==0) {
         my $content = join '', <STDIN>;
         $content = Load($content);
         LOGDIE "Invalid yaml content in $method" unless $content;
-        push @args, $content;
+        push @_, $content;
     }
 
     # Finally, run :
     for my $arg (@extra_args) {
         my $obj;
         if ($have_filenames) {
-            $obj = $client->$method(@args, _load_yaml($arg));
+            $obj = $client->$method(@_, _load_yaml($arg));
         } else {
-            $obj = $client->$method(@args);
+            $obj = $client->$method(@_);
         }
         ERROR $client->errorstring if $client->errorstring;
         next unless $obj;
@@ -215,7 +226,7 @@ sub run {
             }
         } elsif (ref $obj eq 'HASH' && keys %$obj == 1 && $obj->{text}) {
             print $obj->{text};
-        } elsif ($client->tx && $client->tx->req->method eq 'POST' && $meta->get("quiet_post")) {
+        } elsif ($client->tx->req->method eq 'POST' && $meta->get("quiet_post")) {
             my $msg = $client->res->code." ".$client->res->default_message;
             my $got = $client->res->json;
             if ($got && ref $got eq 'HASH' and keys %$got==1 && $got->{text}) {
@@ -223,7 +234,7 @@ sub run {
             }
             INFO $msg;
         } else {
-           print _prettyDump($obj) unless $TESTING;
+           print _prettyDump($obj);
         }
     }
     return;
@@ -237,76 +248,3 @@ sub _prettyDump {
 
 
 1;
-
-__END__
-=pod
-
-=head1 NAME
-
-Clustericious::Client::Command - Command line type processing for clients.
-
-=head1 VERSION
-
-version 0.83_01
-
-=head1 SYNOPSIS
-
-# in fooclient :
-
- use Foo::Client;
- use Clustericious::Client::Command;
-
- Clustericious::Client::Command->run(Foo::Client->new, @ARGV);
-
-Then
-
- fooclient status
- fooclient --trace root status
- fooclient version
- fooclient foobject 31
- fooclient foobject_search --color beige
-
-=head1 DESCRIPTION
-
-This will try to take command line arguments and call the right client
-methods.
-
-Calling C<fooclient bar baz> is equivalent to 
-C<Foo::Client-E<gt>new()-E<gt>bar("baz")>.
-
-=head1 CAVEATS
-
-There are currently a few heuristics used when one of the arguments
-is a filename (i.e. is it a YAML file that should be parsed and send
-as a hashref, or a filename that should be PUT?  Should STDIN be
-used?).  These need to be formalized and documented.
-
-=head1 NOTES
-
-This is a beta release, the API is subject to change without notice.
-
-=head1 METHODS
-
-=head2 C<run>
-
- Clustericious::Client::Command->run(Some::Clustericious::Client->new, @ARGV);
-
-=head1 AUTHOR
-
-original author: Curt Tilmes
-
-current maintainer: Graham Ollis <plicease@cpan.org>
-
-contributors:
-
-Brian Duggan
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2013 by NASA GSFC.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
-=cut
-
